//@line 36 "/builds/tinderbox/Sb-Trunk/Linux_2.4.21-32.0.1.EL_Depend/mozilla/calendar/locales/en-US/sunbird-l10n.js"

//@line 38 "/builds/tinderbox/Sb-Trunk/Linux_2.4.21-32.0.1.EL_Depend/mozilla/calendar/locales/en-US/sunbird-l10n.js"

//@line 41 "/builds/tinderbox/Sb-Trunk/Linux_2.4.21-32.0.1.EL_Depend/mozilla/calendar/locales/en-US/sunbird-l10n.js"
pref("calendar.week.start", 0);

//@line 44 "/builds/tinderbox/Sb-Trunk/Linux_2.4.21-32.0.1.EL_Depend/mozilla/calendar/locales/en-US/sunbird-l10n.js"
pref("calendar.week.d0sundaysoff", true);
pref("calendar.week.d1mondaysoff", false);
pref("calendar.week.d2tuesdaysoff", false);
pref("calendar.week.d3wednesdaysoff", false);
pref("calendar.week.d4thursdaysoff", false);
pref("calendar.week.d5fridaysoff", false);
pref("calendar.week.d6saturdaysoff", true);

pref("general.useragent.locale", "en-US");

//@line 55 "/builds/tinderbox/Sb-Trunk/Linux_2.4.21-32.0.1.EL_Depend/mozilla/calendar/locales/en-US/sunbird-l10n.js"
pref("calendar.categories.names", "Anniversary,Birthday,Business,Calls,Clients,Competition,Customer,Favorites,Follow up,Gifts,Holidays,Ideas,Issues,Miscellaneous,Personal,Projects,Public Holiday,Status,Suppliers,Travel,Vacation");
